/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.api.configuration

import dk.nsp.api.util.ApiHeadersProvider
import dk.nsp.api.util.extensions.asyncIO
import dk.nsp.models.CheckListItem
import dk.nsp.models.DateOfBirth
import dk.nsp.models.PDFIndicator
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.tryOrFailed
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred
import java.time.LocalDateTime

class ConfigurationControllerImpl(private val api: ConfigurationApi) : IConfigurationController {
    override fun updateChecklistItem(
        scope: CoroutineScope,
        episodeOfCareId: String,
        checklistId: String,
        updateChecklistBody: CheckListItem
    ): Deferred<ResultWrapper<CheckListItem>> =
        scope.asyncIO {
            tryOrFailed {
                api.updateChecklistItem(
                    ApiHeadersProvider().getEpisodeOfCareIdentifierHeader(
                        episodeOfCareId
                    ), checklistId, updateChecklistBody
                )
            }
        }

    override fun getCheckedChecklistItems(
        scope: CoroutineScope,
        episodeOfCareId: String
    ): Deferred<ResultWrapper<List<String>>> =
        scope.asyncIO {
            tryOrFailed {
                api.getCheckedChecklistItems(
                    ApiHeadersProvider().getEpisodeOfCareIdentifierHeader(
                        episodeOfCareId
                    )
                )
            }
        }

    override fun getPdfIndicator(
        scope: CoroutineScope,
        episodeOfCareId: String
    ): Deferred<ResultWrapper<PDFIndicator>> = scope.asyncIO {
        tryOrFailed {
            api.getPdfIndicator(
                ApiHeadersProvider().getEpisodeOfCareIdentifierHeader(
                    episodeOfCareId
                )
            )
        }
    }

    override fun updatePdfIndicator(
        scope: CoroutineScope,
        episodeOfCareId: String,
        pdfIndicator: PDFIndicator
    ): Deferred<ResultWrapper<PDFIndicator>> = scope.asyncIO {
        tryOrFailed {
            api.updatePdfIndicator(
                episodeOfCareHeader = ApiHeadersProvider().getEpisodeOfCareIdentifierHeader(
                    episodeOfCareId
                ),
                body = pdfIndicator
            )
        }
    }

    override fun getDateOfBirth(
        scope: CoroutineScope,
        episodeOfCareId: String
    ): Deferred<ResultWrapper<DateOfBirth>> = scope.asyncIO {
        tryOrFailed {
            api.getDateOfBirth(
                ApiHeadersProvider().getEpisodeOfCareIdentifierHeader(
                    episodeOfCareId
                )
            )
        }
    }

    override fun updateDateOfBirth(
        scope: CoroutineScope,
        episodeOfCareId: String,
        newDate: LocalDateTime?
    ): Deferred<ResultWrapper<DateOfBirth>> = scope.asyncIO {
        tryOrFailed {
            api.updateDateOfBirth(
                ApiHeadersProvider().getEpisodeOfCareIdentifierHeader(
                    episodeOfCareId
                ),
                DateOfBirth(newDate)
            )
        }
    }
}