/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.api.datacard

import dk.nsp.api.util.extensions.asyncIO
import dk.nsp.models.*
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.Success
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred
import java.time.LocalDateTime
import java.time.temporal.ChronoUnit

class DataCardControllerMock : IDataCardController {

    private val firstDayOfLastPeriod = LocalDateTime.of(2020, 11, 12, 12, 0)
    private val expectedDateOfDeliveryFromUltraSound = LocalDateTime.of(2021, 8, 14, 12, 0)

    private fun calcTrimester(): Int {
        return when (ChronoUnit.WEEKS.between(firstDayOfLastPeriod, LocalDateTime.now()).toInt()) {
            in (27..50) -> 3
            in (12..27) -> 2
            in (0..12) -> 1
            else -> 0
        }
    }

    private val trimester = calcTrimester()
    private val monthsIntoPregnancy =
        ChronoUnit.MONTHS.between(firstDayOfLastPeriod, LocalDateTime.now()).toDouble()
    private val pregnancyWeek = 37
    private val gestationAge = "${pregnancyWeek - 1}+4"

    private val author = HealthCareOrganization("Org", "sordId")


    override fun getMaternityDataCard(scope: CoroutineScope): Deferred<ResultWrapper<List<DataCard>>> =
        scope.asyncIO {
            Success(
                listOf(
                    DataCard(
                        modifiedTime = LocalDateTime.now(),
                        author = author,
                        recordTarget = CprIdentifiedPerson(
                            PersonName(
                                listOf(
                                    "Maja",
                                    "Ingrid"
                                ), listOf(), "Petersen"
                            ), "0101010102"
                        ),
                        childPartnerSection = ChildPartnerSection(
                            LocalDateTime.now(),
                            author,
                            PersonName(listOf("Jens"), listOf(), "Petersen"),
                            "0101010101"
                        ),
                        languageSection = null,
                        bodyMeasuresSection = null,
                        socialHistorySection = null,
                        previousBirthsSection = null,
                        previousAbortionsSection = null,
                        expectedDateOfDeliverySection = ExpectedDateOfDeliverySection(
                            LocalDateTime.now(),
                            author,
                            trimester,
                            monthsIntoPregnancy,
                            false,
                            ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod(
                                LocalDateTime.now().minusMonths(1),
                                ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriodMenstrualCycle(
                                    IntegerValueWithUnitEnum(20, UnitEnum.D)
                                ),
                                LocalDateTime.now().plusMonths(8),
                                ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod.Certainty.Certain
                            ),
                            expectedDateOfDeliveryFromUltraSound,
                            gestationAge,
                            pregnancyWeek,
                            "No remarks..."
                        ),
                        prenatalRiskAssessmentSection = PrenatalRiskAssessmentSection(
                            PreliminaryRiskAssessment(
                                LocalDateTime.now(),
                                author,
                                null,
                                null,
                                false,
                                true,
                                true,
                                true
                            )
                        ),
                        workEnvironmentImpactSection = null,
                        allergiesSection = AllergiesSection(
                            LocalDateTime.now(),
                            HealthCareOrganization("Org", "sorId"),
                            false,
                            "Nødder, morfin, skaldyr",
                            AllergiesSection.AllergicPredispositionOfChild.DoublePredisposition
                        ),
                        screeningsSection = null,
                        chronicDiseasesSection = null,
                        medicineSection = null,
                        mmrVaccinationSection = null,
                        historyOfPreviousTreatmentWithImpactOnPregnancySection = null,
                        tobaccoUseAndExposureSection = null,
                        alcoholDrinkingBehaviorSection = null,
                        assistedFertilizationSection = null,
                        substanceUseSection = null,
                        dietaryAndExerciseSection = null,
                        healthInsuranceSection = null,
                        birthPlaceSection = null,
                        midwifeClinicSection = null,
                        postpartumNursingSection = null,
                        overallAssessmentSection = OverallAssessmentSection(
                            LocalDateTime.now(),
                            author,
                            OverallAssessmentSectionAllOfPrenatalCareAssessment(
                                "I sidste graviditet præeklampsi, men kontrolleret bagefter og har normal BT. Dårlig fødselsoplevelse efter akut sectio sidst. Overvejer sectio denne fødsel. Faderen arbejder delvist i udlandet så er meget alene, men har godt netværk.",
                                "Here are countless ways you can turn your grilling not only into. Without a doubt there is something very.",
                                1,
                                OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevel.Basic
                            ),
                            ObstetricAssessment(YesNoEnum.True, "Det er der bare behov for"),
                            SocialServicesPerinatalCareAssessment(
                                YesNoEnum.True,
                            )
                        ),
                        personalDataSection = PersonalDataSection(
                            LocalDateTime.now(),
                            author,
                            null,
                            PersonName(
                                listOf(
                                    "Maja",
                                    "Ingrid"
                                ), listOf(), "Petersen"
                            ),
                            "0101010101",
                            Address("Møllegangen 3", "Risskov", "8240"),
                            null,
                            Telecoms("12345678", null, null),
                            null,
                            listOf(
                                PersonalDataSectionAllOfNextOfKin(
                                    "Sarah Hansen",
                                    PersonalDataSectionAllOfNextOfKin.Relation.SSkende,
                                    Telecoms("87654321", null, null),
                                    "Ingen ekstra information :)"
                                )
                            )
                        ),
                        healthcareSection = null,
                        episodeOfCareIdentifier = "episodeOfCareIdentifier",
                        version = "Identifier",
                        dirty = null,
                        documentUniqueId = null,
                        documentStatus = DataCard.DocumentStatus.CLOSED
//                        documentStatus = DataCard.DocumentStatus.ACTIVE
                    ),
                    // Old datacard
                    DataCard(
                        modifiedTime = LocalDateTime.now(),
                        author = HealthCareOrganization("Org", "sordId"),
                        recordTarget = CprIdentifiedPerson(
                            PersonName(
                                listOf(
                                    "Maja",
                                    "Secondary"
                                ), listOf(), "Petersen"
                            ), "0101010102"
                        ),
                        childPartnerSection = ChildPartnerSection(
                            LocalDateTime.now(),
                            author,
                            PersonName(listOf("Miguel"), listOf("de"), "Nada"),
                            "0101010101"
                        ),
                        languageSection = null,
                        bodyMeasuresSection = null,
                        socialHistorySection = null,
                        previousBirthsSection = null,
                        previousAbortionsSection = null,
                        expectedDateOfDeliverySection = ExpectedDateOfDeliverySection(
                            LocalDateTime.now(),
                            author,
                            2,
                            3.5,
                            false,
                            ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod(
                                LocalDateTime.now().minusMonths(1),
                                ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriodMenstrualCycle(
                                    IntegerValueWithUnitEnum(20, UnitEnum.D)
                                ),
                                LocalDateTime.now().plusMonths(8),
                                ExpectedDateOfDeliverySectionAllOfExpectedDateOfDeliveryFromLastPeriod.Certainty.Certain
                            ),
                            null,
                            "15+3",
                            16,
                            "No remarks..."
                        ),
                        prenatalRiskAssessmentSection = null,
                        workEnvironmentImpactSection = null,
                        allergiesSection = AllergiesSection(
                            LocalDateTime.now(),
                            author,
                            false,
                            "Nødder, morfin, skaldyr",
                            AllergiesSection.AllergicPredispositionOfChild.DoublePredisposition
                        ),
                        screeningsSection = null,
                        chronicDiseasesSection = null,
                        medicineSection = null,
                        mmrVaccinationSection = null,
                        historyOfPreviousTreatmentWithImpactOnPregnancySection = null,
                        tobaccoUseAndExposureSection = null,
                        alcoholDrinkingBehaviorSection = null,
                        assistedFertilizationSection = null,
                        substanceUseSection = null,
                        dietaryAndExerciseSection = null,
                        healthInsuranceSection = null,
                        birthPlaceSection = null,
                        midwifeClinicSection = null,
                        postpartumNursingSection = null,
                        overallAssessmentSection = OverallAssessmentSection(
                            LocalDateTime.now(),
                            author,
                            OverallAssessmentSectionAllOfPrenatalCareAssessment(
                                "I sidste graviditet præeklampsi, men kontrolleret bagefter og har normal BT. Dårlig fødselsoplevelse efter akut sectio sidst. Overvejer sectio denne fødsel. Faderen arbejder delvist i udlandet så er meget alene, men har godt netværk.",
                                "Here are countless ways you can turn your grilling not only into. Without a doubt there is something very.",
                                2,
                                OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevel.Basic
                            ),
                            ObstetricAssessment(YesNoEnum.True, "Det er der bare behov for"),
                            SocialServicesPerinatalCareAssessment(
                                YesNoEnum.True,
                            )
                        ),
                        personalDataSection = null,
                        healthcareSection = null,
                        episodeOfCareIdentifier = "episodeOfCareIdentifier",
                        version = "Identifier",
                        dirty = null,
                        documentUniqueId = null,
                        documentStatus = DataCard.DocumentStatus.CLOSED
                    )
                )
            )
        }
}