/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.appointments

import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.*
import androidx.navigation.NavDirections
import com.commonsense.android.kotlin.base.extensions.collections.categorizeInto
import com.commonsense.android.kotlin.base.extensions.coroutines.launchIO
import dk.nsp.domain.careplan.CareplanActivities
import dk.nsp.domain.careplan.CareplanActivitiesMonth
import dk.nsp.domain.careplan.CareplanActivityYear
import dk.nsp.domain.repository.CareplanRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.adapterviews.EmptyStateFullHeight
import dk.nsp.mingraviditet.ui.main.appointments.details.ActivityNavData
import dk.nsp.mingraviditet.ui.main.appointments.rows.*
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.*
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.models.Activity
import dk.nsp.models.CarePlan
import dk.nsp.util.TextModel
import java.time.LocalDateTime

class AppointmentsViewModel @ViewModelInject constructor(
    private val careplanRepository: CareplanRepository
) : ViewModel() {

    private val _careplansMapped: MutableLiveData<CareplanActivities> = MutableLiveData()

    val rowsToShow: LiveData<List<RecyclerAdapterView<*>>> = _careplansMapped.transform {
        if (it.pastActivities.isEmpty() && it.futureActivities.isEmpty()) {
            createEmptyState()
        } else {
            mapCareplanActivitiesToRows(it)
        }
    }

    private val _navigateTo = MutableLiveData<SingleEvent<NavDirections>>()
    val navigateTo: LiveData<SingleEvent<NavDirections>> = _navigateTo

    private val careplanObserver = Observer<CarePlan>() { getCarePlans(it) }

    init {
        careplanRepository.selectedCareplan.observeForever(careplanObserver)
    }

    override fun onCleared() {
        super.onCleared()

        careplanRepository.selectedCareplan.removeObserver(careplanObserver)
    }

    private fun getCarePlans(careplan: CarePlan) = viewModelScope.launchIO {
        sortActivitiesIntoModel(careplan)
    }

    private fun sortActivitiesIntoModel(careplan: CarePlan) =
        viewModelScope.launchIO {

            // Sort all activities into past and future:
            val activities = careplan.activities ?: listOf()
            val (past, future) = activities.categorizeInto(
                { it.sortDate.isBeforeNow() },
                { it.sortDate.isAfterNow() })

            //Make past section first
            val pastActivites = mapActivitiesIntoActivityYears(past)

            // Make future section afterwards:
            val futureActivities = mapActivitiesIntoActivityYears(future)

            //Final product:
            _careplansMapped.postValue(CareplanActivities(pastActivites, futureActivities))
        }

    private fun mapActivitiesIntoActivityYears(activities: List<Activity>): List<CareplanActivityYear> {
        val yearSet = activities.map { it.sortDate?.year }.toSet()

        return yearSet.map { year ->
            val activitiesForYear = activities.filter { it.sortDate?.year == year }
            val monthsOfActivityYear =
                activitiesForYear.map { it.sortDate?.month }.toSet()

            val careplanActivitiesMonth = monthsOfActivityYear.map { month ->
                val activitiesForMonth =
                    activities.filter { it.sortDate?.year == year && it.sortDate?.month == month }
                CareplanActivitiesMonth(

                    LocalDateTime.of(
                        year ?: LocalDateTime.now().year,
                        month?.value ?: 1,
                        1,
                        0,
                        0
                    ), activitiesForMonth
                )
            }

            CareplanActivityYear(
                LocalDateTime.now().createWithYear(
                    year ?: LocalDateTime.now().month.value
                ), careplanActivitiesMonth
            )
        }
    }

    private fun mapCareplanActivitiesToRows(it: CareplanActivities): List<RecyclerAdapterView<*>> {
        val rows = mutableListOf<RecyclerAdapterView<*>>()

        // Past section:
        rows.add(AppointmentInfoRow(isPast = true))
        it.pastActivities.forEach { activityYear ->
            rows.add(AppointmentsYearRow(activityYear.year.year.toString(), true))

            activityYear.monthActivities.forEach { activityMonth ->
                rows.add(AppointmentsMonthRow(activityMonth.month.toLongMonthFormat(), true))

                val activitiesForMonth = activityMonth.activites.map { activity ->
                    AppointmentRow(AppointmentRowBindingModel(activity) {
                        triggerActivityNavigation(activity)
                    })
                }
                rows.addAll(activitiesForMonth)
            }
        }
        //If anything is previous we add the header
        if (rows.isNotEmpty()) {
            rows.add(AppointmentsPastHeaderRow())
        }
        // always add header for future appointments
        rows.add(AppointmentsHeaderRow())

        it.futureActivities.forEach { year ->
            rows.add(AppointmentsYearRow(year.year.year.toString(), false))

            year.monthActivities.forEach { month ->
                rows.add(AppointmentsMonthRow(month.month.toLongMonthFormat(), false))

                val activitiesForMonth = month.activites.map { activity ->
                    AppointmentRow(AppointmentRowBindingModel(activity) {
                        triggerActivityNavigation(activity)
                    })
                }
                rows.addAll(activitiesForMonth)
            }
        }
        rows.add(AppointmentInfoRow(isPast = false))
        return rows
    }

    private fun triggerActivityNavigation(activity: Activity) {
        _navigateTo.postValue(
            AppointmentsFragmentDirections.actionKnowledgeFragmentToCareplanActivityDetailsFragment(
                ActivityNavData.WithActivity(activity)
            ).toSingleEvent()
        )
    }

    private fun createEmptyState(): List<RecyclerAdapterView<*>> {
        return listOf(
            EmptyStateFullHeight(
                TextModel.from(R.string.careplan_activities_empty_state_title),
                TextModel.from(R.string.careplan_activities_empty_state_description)
            )
        )
    }
}