/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.journal

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.constraintlayout.motion.widget.MotionLayout
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentJournalBinding
import dk.nsp.mingraviditet.ui.widgets.BottomInfoToggler
import dk.nsp.mingraviditet.util.android.motionlayout.MotionLayoutStateEnum
import dk.nsp.mingraviditet.util.android.recycler.DelegatingListAdapter
import dk.nsp.mingraviditet.util.extensions.handleToolbarState
import dk.nsp.mingraviditet.util.extensions.navigateSafe
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled
import dk.nsp.mingraviditet.util.extensions.setupToolbarStateListener

@AndroidEntryPoint
class JournalFragment : Fragment() {
    private lateinit var binding: FragmentJournalBinding

    private val adapter by lazy {
        DelegatingListAdapter()
    }

    private val journalViewModel: JournalViewModel by viewModels()

    private var state: MotionLayoutStateEnum = MotionLayoutStateEnum.Start
        set(value) {
            field = value
            (binding.root as MotionLayout).handleToolbarState(value)
        }

    private var infoToggler: BottomInfoToggler? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentJournalBinding.inflate(inflater, null, false)
        binding.toolbarTitle.setText(R.string.bottom_navigation_title_journal)
        binding.recyclerview.adapter = adapter

        setupToolbarState()

        infoToggler = BottomInfoToggler(binding.bottomInfo, binding.recyclerview)

        journalViewModel.sectionCards.observe(viewLifecycleOwner) {
            adapter.submitList(it)
        }

        journalViewModel.onNavigateToSection.observeIfNotHandled(viewLifecycleOwner) {
            when (it) {
                JournalSection.Screenings,
                JournalSection.AroundThePregnancy,
                JournalSection.HealthCare,
                JournalSection.PersonalData -> {
                    findNavController().navigateSafe(
                        JournalFragmentDirections.actionJournalFragmentToJournalSectionOverviewFragment(
                            it
                        )
                    )
                }
                JournalSection.OverallAssessment -> {
                    findNavController().navigateSafe(JournalFragmentDirections.actionJournalFragmentToJournalOverallAssessmentFragment())
                }
                JournalSection.JournalNotes -> {
                    findNavController().navigateSafe(JournalFragmentDirections.actionJournalFragmentToJournalNotesFragment())
                }
                JournalSection.Scans -> {
                    findNavController().navigateSafe(JournalFragmentDirections.actionJournalFragmentToJournalScansFragment())
                }
                JournalSection.Measurements -> {
                    findNavController().navigateSafe(JournalFragmentDirections.actionJournalFragmentToJournalMeasurementsFragment())
                }
            }
        }

        return binding.root
    }

    private fun setupToolbarState() {
        setupToolbarStateListener(binding.root) { newState ->
            state = newState
        }

        (binding.root as MotionLayout).handleToolbarState(state)
    }
}