/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.more.settings

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.commonsense.android.kotlin.system.extensions.onBackPressed
import com.google.android.material.snackbar.Snackbar
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentPregnancyDataAccessBinding
import dk.nsp.mingraviditet.domain.AlertDialogButtonModel
import dk.nsp.mingraviditet.domain.AlertDialogViewModel
import dk.nsp.mingraviditet.util.extensions.createAndShowWith
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled

@AndroidEntryPoint
class DataAccessFragment : Fragment() {

    private val vm by viewModels<PregnancyDataAccessViewModel>()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        val binding = FragmentPregnancyDataAccessBinding.inflate(inflater, container, false)
        binding.toolbar.setNavigationOnClickListener { onBackPressed() }


        binding.root.post {
            //Set onCheckedChange on post, so we don't get a callback from the initial toggle
            binding.receiveDataToggle.setOnCheckedChangeListener { _, isChecked ->
                if (!isChecked) {
                    confirmDisablePDFIndicator(binding)
                } else {
                    vm.updatePdfIndicator(isChecked)
                }
            }
        }

        vm.receiveDataToggleValue.observe(viewLifecycleOwner) {
            binding.receiveDataToggle.isChecked = it
        }

        vm.pdfIndicatorFailed.observeIfNotHandled(viewLifecycleOwner) {
            binding.receiveDataToggle.isEnabled = false
        }

        vm.onError.observeIfNotHandled(viewLifecycleOwner) {
            it.getString(binding.root.resources)?.let { errorText ->
                Snackbar.make(
                    binding.root,
                    errorText,
                    Snackbar.LENGTH_LONG
                ).show()
            }
        }

        vm.onSendDataToDigitalMailClicked.observeIfNotHandled(viewLifecycleOwner) {
            Snackbar.make(binding.root, "Ikke implementeret i BFF", Snackbar.LENGTH_LONG).show()
        }

        binding.vm = vm
        return binding.root
    }

    private fun confirmDisablePDFIndicator(
        binding: FragmentPregnancyDataAccessBinding
    ) {
        AlertDialog.Builder(binding.root.context).createAndShowWith(
            AlertDialogViewModel(
                title = R.string.settings_data_access_receive_data_disable_dialog_title,
                message = R.string.settings_data_access_receive_data_disable_dialog_description,
                cancelable = false,
                positiveButton = AlertDialogButtonModel(R.string.common_ok) { _, _ ->
                    vm.updatePdfIndicator(wantsPdf = false)
                },
                negativeButton = AlertDialogButtonModel(R.string.common_cta_close) { _, _ ->
                    //Reset state
                    binding.receiveDataToggle.isChecked = true
                }
            )
        )
    }
}