/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.more.settings

import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dk.nsp.domain.repository.DataAccessRepository
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.util.extensions.postSingleEvent
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.models.DataCard
import dk.nsp.models.PDFIndicator
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.util.asyncDefault
import dk.nsp.util.TextModel


class PregnancyDataAccessViewModel @ViewModelInject constructor(
    val dataCardRepository: DataCardRepository,
    private val dataAccessRepository: DataAccessRepository
) : ViewModel() {

    private val _onError: MutableLiveData<SingleEvent<TextModel>> = MutableLiveData()
    val onError: LiveData<SingleEvent<TextModel>> = _onError

    private val _pdfIndicatorFailed = MutableLiveData<SingleEvent<Unit>>()
    val pdfIndicatorFailed: LiveData<SingleEvent<Unit>> = _pdfIndicatorFailed

    private val _pdfIndicator = MutableLiveData<PDFIndicator?>(null)

    val receiveDataToggleValue = MediatorLiveData<Boolean>().apply {
        addSource(dataCardRepository.currentDataCard) { dataCard ->
            getPdfIndicator(dataCard)
        }
        addSource(_pdfIndicator) {
            postValue(it?.wantsPDF == true)
        }
    }

    private fun MediatorLiveData<Boolean>.getPdfIndicator(dataCard: DataCard?) =
        viewModelScope.asyncDefault {
            dataCard?.episodeOfCareIdentifier?.let { episodeOfCareId ->
                when (val pdfIndicator =
                    dataAccessRepository.getPdfIndicator(this, episodeOfCareId).await()) {
                    is Success -> {
                        postValue(pdfIndicator.value.wantsPDF == true)
                    }
                    is Failed -> {
                        _pdfIndicatorFailed.postSingleEvent()
                        _onError.postSingleEvent(TextModel.from(R.string.settings_data_access_receive_data_backend_error))
                        postValue(false)
                    }
                }
            }
        }

    fun updatePdfIndicator(wantsPdf: Boolean) {
        val episodeOfCareId =
            dataCardRepository.currentDataCard.value?.episodeOfCareIdentifier ?: return
        //Don't run if value isn't set by backend yet
        if (wantsPdf != _pdfIndicator.value?.wantsPDF) {
            viewModelScope.asyncDefault {
                when (val setPDFIndicator =
                    dataAccessRepository.setPdfIndicator(this, episodeOfCareId, wantsPdf).await()) {
                    is Success -> {
                        _pdfIndicator.postValue(setPDFIndicator.value)
                    }
                    is Failed -> _pdfIndicator.postValue(null)
                }
            }
        }
    }

    private val _onSendDataToDigitalMailClicked = MutableLiveData<SingleEvent<Unit>>()
    val onSendDataToDigitalMailClicked: LiveData<SingleEvent<Unit>> =
        _onSendDataToDigitalMailClicked

    // TODO: 3/16/21 Do this, when we have an endpoint
    fun sendDataToDigitalMail() {
        _onSendDataToDigitalMailClicked.postSingleEvent()
    }
}