/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.overview

import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.commonsense.android.kotlin.base.extensions.coroutines.launchIO
import dk.nsp.domain.knowledge.Article
import dk.nsp.domain.knowledge.ArticleCategory
import dk.nsp.domain.repository.ArticlesRepository
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.extensions.getOverviewPregnancyState
import dk.nsp.mingraviditet.extensions.userName
import dk.nsp.mingraviditet.ui.adapterviews.EmptyStateRow
import dk.nsp.mingraviditet.ui.common.SectionHeaderAdapterView
import dk.nsp.mingraviditet.ui.main.knowledge.CategoryCard
import dk.nsp.mingraviditet.ui.main.knowledge.viewbinding.CategoryCardBindingModel
import dk.nsp.mingraviditet.ui.main.overview.widget.OverviewPregnancyState
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.postSingleEvent
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.extensions.transform
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.models.ArticleSearchResult
import dk.nsp.models.DataCard
import dk.nsp.models.DateOfBirth
import dk.nsp.shared.Failed
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.Success
import dk.nsp.util.TextModel
import dk.nsp.util.hasSearchResults
import dk.nsp.util.searchResults
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.async
import timber.log.Timber
import java.time.LocalDateTime

class OverviewFragmentViewModel @ViewModelInject constructor(
    private val articlesRepository: ArticlesRepository,
    private val dataCardRepository: DataCardRepository
) : ViewModel() {

    private val _rowsToShow = MutableLiveData<List<RecyclerAdapterView<*>>>()
    val rowsToShow: LiveData<List<RecyclerAdapterView<*>>> = _rowsToShow

    private val _onFeedbackClicked = MutableLiveData<SingleEvent<Unit>>()
    val onFeedbackClicked: LiveData<SingleEvent<Unit>> = _onFeedbackClicked

    //TODO Virker ikke med string, kun med Unit...
    private val _onUpdateDateOfBirthClicked = MutableLiveData<SingleEvent<DateOfBirth>>()
    val onUpdateDateOfBirthClicked: LiveData<SingleEvent<DateOfBirth>> = _onUpdateDateOfBirthClicked

    private val _onSearchedArticleClicked: MutableLiveData<SingleEvent<Pair<Article, Int>>> =
        MutableLiveData()
    val onSearchArticleClicked: LiveData<SingleEvent<Pair<Article, Int>>> =
        _onSearchedArticleClicked

    private val _navigateToKnowledge = MutableLiveData<SingleEvent<Unit>>()
    val navigateToKnowledge: LiveData<SingleEvent<Unit>> = _navigateToKnowledge

    private val pregnancyWeek =
        dataCardRepository.currentDataCard.transform { currentDataCard: DataCard? ->
            // Default to -1 if no calculated pregnancy week was found
            currentDataCard?.expectedDateOfDeliverySection?.week ?: -1
        }

    val usersName = dataCardRepository.currentDataCard.transform {
        it?.userName
            ?: "Navnebeskyttet" // TODO: 3/5/21 SDS ville helst ikke have noget "navnbeskyttet"
    }

    private val pregnancyWeekChangedObserver = Observer<Int> { pregnancyWeek ->
        if (pregnancyWeek != -1) {
            updateArticleCards(pregnancyWeek)
        } else {
            // TODO: 2/24/21 Flowerpot ?
            Timber.e("No pregnancyWeek :( $pregnancyWeek")
        }
    }

    val pregnancyProgress = MediatorLiveData<OverviewPregnancyState>().apply {
        addSource(dataCardRepository.currentDataCard) {
            postValue(updatePregnancyState(
                it,
                dataCardRepository.dateOfBirthForCurrentDataCard.value
            ))
        }
        addSource(dataCardRepository.dateOfBirthForCurrentDataCard) {
            postValue(
                updatePregnancyState(
                    dataCardRepository.currentDataCard.value,
                    it
                )
            )
        }
    } as LiveData<OverviewPregnancyState>

    init {
        pregnancyWeek.observeForever(pregnancyWeekChangedObserver)
    }

    override fun onCleared() {
        super.onCleared()
        pregnancyWeek.removeObserver(pregnancyWeekChangedObserver)
    }

    private fun onDidGiveBirthPressed() {
        _onUpdateDateOfBirthClicked.postSingleEvent(DateOfBirth())
    }

    private fun presentDateOfBirthCallback(currentDateOfBirth: LocalDateTime?) {
        _onUpdateDateOfBirthClicked.postSingleEvent(DateOfBirth(currentDateOfBirth))
    }

    private fun updatePregnancyState(currentCard: DataCard?, dateOfBirth: LocalDateTime?) =
        currentCard.getOverviewPregnancyState(
            dateOfBirth,
            ::onDidGiveBirthPressed
        ) {
            presentDateOfBirthCallback(dateOfBirth)
        }


    private fun updateArticleCards(pregnancyWeek: Int) {
        viewModelScope.launchIO {
            val thisWeek = async { getArticlesForWeek(this, pregnancyWeek) }
            val nextWeek = async { getArticlesForWeek(this, pregnancyWeek + 1) }

            val combine = mutableListOf<RecyclerAdapterView<*>>()
            combine.addAll(createArticleSection(thisWeek.await(), false))
            combine.addAll(createArticleSection(nextWeek.await(), true))

            combine.add(createFeedbackCard())

            _rowsToShow.postValue(combine)
        }
    }

    private fun createFeedbackCard() = OverviewFeedbackCard { _onFeedbackClicked.postSingleEvent() }

    private fun createSectionHeader(isFutureSection: Boolean): RecyclerAdapterView<*> {
        val moreButtonText =
            if (!isFutureSection) TextModel.from(R.string.overview_find_more_knowledge_button) else null
        val moreButtonAction = if (!isFutureSection) {
            { _navigateToKnowledge.postSingleEvent() }
        } else null

        return SectionHeaderAdapterView(
            TextModel.from(
                if (isFutureSection) R.string.overview_relevant_next_week_title else R.string.overview_relevant_right_now_title
            ),
            moreButtonText,
            moreButtonAction
        )
    }

    private fun createArticleSection(
        result: ResultWrapper<List<ArticleCategory>>,
        isFutureSection: Boolean
    ): List<RecyclerAdapterView<*>> {
        val rowsToShow = mutableListOf<RecyclerAdapterView<*>>()

        when (result) {
            is Failed -> return listOf(
                createSectionHeader(isFutureSection),
                EmptyStateRow(
                    TextModel.Companion.from(R.string.common_alert_title_error),
                    TextModel.Companion.from("${result.error.message}")
                )
            )
            is Success -> {
                if (!result.value.hasSearchResults() && !isFutureSection) {
                    rowsToShow.add(createSectionHeader(isFutureSection))
                    rowsToShow.add(
                        EmptyStateRow(
                            TextModel.from(R.string.overview_empty_state_title),
                            TextModel.from(R.string.overview_empty_state_text)
                        )
                    )
                } else {
                    rowsToShow.add(createSectionHeader(isFutureSection))
                    result.value.map {
                        val cards = it.searchResults().map {
                            createCardForSearchResult(it)
                        }
                        rowsToShow.addAll(cards)
                    }
                }

                return rowsToShow
            }
        }
    }

    private suspend fun getArticlesForWeek(
        scope: CoroutineScope,
        week: Int
    ): ResultWrapper<List<ArticleCategory>> {
        return articlesRepository.getArticlesForWeek(scope, week).await()
    }

    private fun createCardForSearchResult(articleSearchResult: ArticleSearchResult): CategoryCard {
        return CategoryCard(
            CategoryCardBindingModel(
                articleSearchResult.title,
                articleSearchResult.description,
                TextModel.from(articleSearchResult.subtitle),
                articleSearchResult.iconUrl
            ) {
                _onSearchedArticleClicked.postValue(
                    Pair(
                        articleSearchResult.article,
                        articleSearchResult.articleSectionId ?: -1
                    ).toSingleEvent()
                )
            })
    }
}