/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.widgets


import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.Path
import android.graphics.PathDashPathEffect
import android.util.AttributeSet
import android.view.View
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.util.extensions.getColorFromAttr


/**
 * This is a custom view to draw dotted, vertical lines.
 *
 * A regular shape drawable (like we use for horizontal lines) is usually not sufficient because
 * rotating such a (by default horizontal) line to be vertical does not recalculate the correct
 * with and height if they are set to match_parent or wrap_content.
 *
 * Furthermore, this view draws actual round dots, not those fake tiny square ones like shape
 * drawables do.
 *
 * A more elaborate version of this view would use custom attributes to set the color of the line
 * more dynamically, as well as the line shape, gap size, etc.
 */
class DottedLine : View {
    private var mPaint: Paint = Paint()

    private var lineColor = context.getColorFromAttr(R.attr.dividerColor)

    constructor(context: Context?) : super(context) {
        init(null, 0)
    }

    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs) {
        init(attrs, 0)
    }

    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context,
        attrs,
        defStyleAttr
    ) {
        init(attrs, defStyleAttr)
    }

    private fun init(attrs: AttributeSet?, defStyle: Int) {
        // Load attributes
        val a = context.obtainStyledAttributes(
            attrs, R.styleable.DottedLine, defStyle, 0
        )

        lineColor = a.getColor(R.styleable.DottedLine_dotted_line_color, lineColor)

        a.recycle()

        mPaint = Paint()
        mPaint.color = lineColor
        val size: Float =
            resources.getDimensionPixelSize(R.dimen.dotted_divider_size_vert).toFloat()
        val gap: Float = resources.getDimensionPixelSize(R.dimen.dotted_divider_gap_vert).toFloat()
        mPaint.style = Paint.Style.FILL


        // To get actually round dots, we define a circle...
        val path = Path()
        path.addRoundRect(
            0f,
            0f,
            size.toFloat() * 4,
            size.toFloat() * 2,
            size.toFloat() * 4,
            size.toFloat() * 4,
            Path.Direction.CCW
        )
//        path.addCircle(0f, 0f, size.toFloat(), Path.Direction.CCW)
        // ...and use the path with the circle as our path effect
        mPaint.pathEffect = PathDashPathEffect(
            path,
            gap.toFloat(),
            0f,
            PathDashPathEffect.Style.ROTATE
        )

    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        val size: Float =
            resources.getDimensionPixelSize(R.dimen.dotted_divider_size_vert).toFloat()
        canvas.drawLine(width / 2 + size, 0f, width / 2 + size, height.toFloat(), mPaint)
    }
}