/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.domain.repository

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import dk.nsp.api.getBackendApiLayer
import dk.nsp.models.Observation
import dk.nsp.models.ObservationTable
import dk.nsp.models.ScanningObservationTable
import dk.nsp.shared.*
import dk.nsp.shared.util.asyncDefault
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred

class ObservationsRepository(context: Context) {
    private val observationsController = context.getBackendApiLayer().observationsController

    private val _observations: MutableLiveData<List<Observation>> = MutableLiveData()
    val observations: LiveData<List<Observation>> = _observations

    private val _clinicalObservations: MutableLiveData<ObservationTable> = MutableLiveData()

    private val _scanObservations: MutableLiveData<ScanningObservationTable> = MutableLiveData()

    fun getObservationsAsync(
        scope: CoroutineScope,
        episodeOfCareIdentifier: String
    ): Deferred<ResultWrapper<List<Observation>>> =
        scope.asyncDefault {
            if (_observations.value == null) {
                val observationsRes =
                    observationsController.getObservations(this, episodeOfCareIdentifier).await()

                observationsRes.mapSuccess {
                    val observationsDomain = it

                    _observations.postValue(observationsDomain)

                    observationsDomain
                }
            } else {
                _observations.value?.let { Success(it) }
                    ?: Failed(Exception("No observation value at the time"))
            }
        }

    fun getClinicalObservationsAsync(
        scope: CoroutineScope,
        episodeOfCareIdentifier: String
    ): Deferred<ResultWrapper<ObservationTable>> =
        scope.asyncDefault {

            val cachedData = _clinicalObservations.value

            cachedData?.toSuccess()
                ?: observationsController.getClinicalObservationsTable(scope, episodeOfCareIdentifier)
                    .await()
        }

    fun getScanObservationsTable(scope: CoroutineScope, episodeOfCareIdentifier: String): Deferred<ResultWrapper<ScanningObservationTable>> = scope.asyncDefault {
        val cachedData = _scanObservations.value

        cachedData?.toSuccess()
            ?: observationsController.getScanObservationsTable(scope, episodeOfCareIdentifier).await()
    }
}