/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.shared.util.sharedprefs

import android.content.Context
import android.content.SharedPreferences
import dk.nsp.shared.tryAndLog

class SharedPrefsHelper(
    context: Context,
    settingsName: String = defaultPreferencesName
) {

    private val sharedPref = context.getSharedPreferences(settingsName, Context.MODE_PRIVATE)

    //region Biometric
    /**
     * should we use fingerprint ?
     */
    var useBiometrics: Boolean?
        get() = get(usesFingerprintKey, false)
        set(value) = storeOrClear(usesFingerprintKey, value)

    /**
     * whenever we have asked the user to use fingerprint
     */
    var askedForUseBiometrics: Boolean?
        get() = get(askedForUseFingerprint, false)
        set(value) = storeOrClear(askedForUseFingerprint, value)

    fun clearBiometricSettings() {
        useBiometrics = null
        askedForUseBiometrics = null
    }
    //endregion

    var allowScreenshots: Boolean?
        get() = get(allowScreenshotsEnabledKey, true)
        set(value) = storeOrClear(allowScreenshotsEnabledKey, value)

    //region Common read/write shorthands
    //region Store functions
    fun store(forKey: String, value: String?) = sharedPref.applyWith { putString(forKey, value) }
    fun store(forKey: String, value: Int) = sharedPref.applyWith { putInt(forKey, value) }
    fun store(forKey: String, value: Long) = sharedPref.applyWith { putLong(forKey, value) }
    fun store(forKey: String, value: Boolean) = sharedPref.applyWith { putBoolean(forKey, value) }
    //endregion

    //region Destructive store
    private fun storeOrClear(forKey: String, value: Boolean?) {
        if (value == null) {
            remove(forKey)
        } else {
            store(forKey, value)
        }
    }
    //endregion

    //region Fetch functions
    fun get(forKey: String, default: String? = null) = sharedPref.getString(forKey, default)
    fun get(forKey: String, default: Int) = sharedPref.getInt(forKey, default)
    fun get(forKey: String, default: Long) = sharedPref.getLong(forKey, default)
    fun get(forKey: String, default: Boolean) = sharedPref.getBoolean(forKey, default)
    //endregion

    fun hasValue(forKey: String) = sharedPref.contains(forKey)

    fun remove(forKey: String) = sharedPref.applyWith { remove(forKey) }
    //endregion

    companion object {
        private const val usesFingerprintKey = "UsesFingerprintPreference"
        private const val askedForUseFingerprint = "AskedForUseFingerprintPreference"
        private const val allowScreenshotsEnabledKey = "screenSecurityEnabledPreference"
        const val defaultPreferencesName = "MinGraviditetAppPrefs"
    }
}


fun Context.getDefaultSharedPrefs() = SharedPrefsHelper(this)

private inline fun SharedPreferences.applyWith(crossinline action: (SharedPreferences.Editor.() -> Unit)) {
    tryAndLog {
        edit().apply(action).apply()
    }
}